<!doctype html>
<html lang="en">

<head>
  <meta charset="utf-8">
  <title>{% block title %}{% endblock title %}</title>
  <meta name="description" content="">
  <meta name="viewport" content="width=device-width, initial-scale=1">

  <link rel="stylesheet" href="/demo/static/normalize.css">
  <link rel="stylesheet" href="/demo/static/skeleton.css">
  <link rel="stylesheet" href="/demo/static/main.css">
</head>

<body>
{% block content %}
  <!-- Add your site or application content here -->
  <p>Hello world! This is HTML5 Boilerplate.</p>

{% endblock content %}
{% block js %}
{% endblock js %}
</body>

</html>

